# Generated by Django 5.1.5 on 2025-08-20 06:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0004_childtype_parenttype_itemtype'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='parenttype',
            name='child',
        ),
        migrations.RemoveField(
            model_name='itemtype',
            name='parenttype',
        ),
        migrations.DeleteModel(
            name='ChildType',
        ),
        migrations.DeleteModel(
            name='ItemType',
        ),
        migrations.DeleteModel(
            name='ParentType',
        ),
    ]
