# Generated by Django 5.2.5 on 2025-08-20 06:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tests", "0002_lazychoicemodel_alter_allfieldsmodel_file_field_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="allfieldsmodel",
            name="many_to_many_field",
            field=models.ManyToManyField(
                blank=True,
                related_name="all_fields_models_m2m",
                through="tests.Membership",
                through_fields=("all_fields_model", "related_model"),
                to="tests.relatedmodel",
            ),
        ),
    ]
